//--------------------------------------------------------------//
// Gloss Effect
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

// transformation matrices
float4x4 mtxWorld	: WORLD;  
float4x4 mtxView	: VIEW;
float4x4 mtxProj	: PROJECTION;
float4   gLight;    
float3   vecView;

texture baseTex;
texture glossTex;

sampler2D baseTex0 = sampler_state
{
	Texture   = <baseTex>;
	MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler2D glossTex0 = sampler_state
{
	Texture   = <glossTex>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
	float3 Pos       : POSITION;
	float3 Normal    : NORMAL;
	float4 Diffuse   : COLOR0;
	float2 Tex0      : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 Pos        : POSITION;  
    float4 Diffuse    : COLOR0; 
    float4 Spec       : COLOR1;
    float2 Tex0       : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------

VS_OUTPUT VtxShader_Gloss( VS_INPUT In )
{
    VS_OUTPUT Out;

	// position xform
	In.Pos   = mul(float4(In.Pos, 1), (float4x3)mtxWorld);
	In.Pos   = mul(float4(In.Pos, 1), (float4x3)mtxView);
	Out.Pos  = mul(float4(In.Pos, 1), mtxProj);
		
	float3 N = normalize( mul( In.Normal, (float3x3)mtxWorld ) );
	float3 V = normalize( vecView - Out.Pos );
	float3 L = normalize( gLight );
	
	// Specular lighting ----	
	float3 R = reflect( L, N );
	R = normalize( R );
	
	float dpS = dot( V,R );
	dpS = max( 0, dpS );
	Out.Spec = pow( dpS, 20 );
	Out.Spec.a = 1.0f;
	// End Spec -------------
	
	// Diffuse lighting -----
	float dp = dot( -L, N );
	dp = max( 0, dp );
	
	Out.Diffuse = dp;
	Out.Diffuse.a = 1.0f;	
	// End Diffuse ----------
	
	// pass-thru texc coordinates
	Out.Tex0 = In.Tex0;
	
    return Out;    
}

//--------------------------------------------------------------------------------------
// Pixel shader
//--------------------------------------------------------------------------------------

float4 PixShader_Gloss( VS_OUTPUT In ) : COLOR
{ 
    float4 baseColour  = tex2D(baseTex0,  In.Tex0);
	float4 glossColour = tex2D(glossTex0, In.Tex0);
	
    baseColour  *= In.Diffuse;
    glossColour *= (In.Spec * 1.5f);

	return ( baseColour + glossColour );
}

float4 PixShader_Room( VS_OUTPUT In ) : COLOR
{ 
    float4 baseColour  = tex2D(baseTex0,  In.Tex0);
	float4 glossColour = tex2D(glossTex0, In.Tex0);
	
	float tmp = 1*In.Diffuse;
	if(tmp == 0)
		In.Diffuse = float4(0.2f,0.2f,0.2f,1.0f);
    baseColour  *= In.Diffuse;
    glossColour *= (In.Spec * 5.0f);

	return ( baseColour + glossColour );

}

//---------------------------------------------------------------------------------------

technique GlossEffect
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader_Gloss( );
		PixelShader  = compile ps_2_0 PixShader_Gloss( );          
    }
}

technique Room
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader_Gloss( );
		PixelShader  = compile ps_2_0 PixShader_Room( );          
    }
}

//---------------------------------------------------------------------------------------