//--------------------------------------------------------------//
// Wood Effect
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

// transformation matrices
float4x4 mtxWorld	: WORLD;  
float4x4 mtxView	: VIEW;
float4x4 mtxProj	: PROJECTION;   
float3 gLight;
float3 vecView;

float scale 		= float( 0.4f );
float diffuseMult	= float( 0.89 );
float specMult		= float( 0.66 );
float4 darkWood 	= float4( 0.0f, 0.00f, 0.30f, 1.00f );
float4 liteWood 	= float4( 1.0f, 0.10f, 0.0f, 1.00f );
float frequency 	= float( 5.72f );
float ringSharpness = float( 100.00f );
float noiseScale 	= float( 5.0f );

texture noiseTex;

sampler Noise = sampler_state
{
	Texture   = <noiseTex>;
	ADDRESSU  = WRAP;
	ADDRESSV  = WRAP;
	ADDRESSW  = WRAP;
	MAGFILTER = LINEAR;
	MINFILTER = LINEAR;
	MIPFILTER = LINEAR;

};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
   float4 Pos		: POSITION;
   float3 normal	: NORMAL;
};

struct VS_OUTPUT 
{
	float4 Pos		: POSITION;
	float3 Pos2		: TEXCOORD1;
	float4 Diffuse	: COLOR0; 
	float4 Spec		: COLOR1;
};

//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------

VS_OUTPUT VtxShader_Wood( VS_INPUT In )
{
    VS_OUTPUT Out;
    
	// position xform    

	In.Pos   = mul(In.Pos, mtxWorld);
	Out.Pos2 = In.Pos;		
	In.Pos   = mul(In.Pos, mtxView);
	Out.Pos  = mul(In.Pos, mtxProj);

    // Pass position to fragment shader
    Out.Pos2 *= scale;

	float3 N = normalize( mul( In.normal, (float3x3)mtxWorld ) );
	float3 V = normalize( vecView - Out.Pos );
	float3 L = normalize( gLight );

	// Specular lighting ----
	float3 R = reflect( L, N );
	R = normalize(R);
	
	float dpS = dot( V,R );
	dpS = max( 0, dpS );
	Out.Spec = pow( dpS, 15 );
	Out.Spec.a = 1.0f;
	// End Spec -------------	
		
	// Diffuse lighting -----
	float dp = dot( -L, N );
	dp = max( 0, dp );
	
	Out.Diffuse = dp;
	Out.Diffuse.a = 1.0f;
	// End Diffuse ----------	
	
    return Out;
}

//--------------------------------------------------------------------------------------
// Pixel shader
//--------------------------------------------------------------------------------------

float4 PixShader_Wood( VS_OUTPUT In ) : COLOR 
{
	// Signed noise
	float snoise = 2 * tex3D(Noise, In.Pos2).r - 1;

	// Rings goes along z axis, perturbed with some noise
	float r = frac(frequency * In.Pos2.z + noiseScale * snoise);
	float invMax = pow(ringSharpness, ringSharpness / (ringSharpness - 1)) / (ringSharpness - 1);
	float ring = invMax * (r - pow(r, ringSharpness));

	// Add some noise and get our base color
	float lrp = ring + snoise;
	float4 base = lerp(darkWood, liteWood, lrp);

	return base* In.Diffuse + In.Spec*specMult;
}

//---------------------------------------------------------------------------------------

technique WoodEffect
{
   pass P0
   {
      VertexShader = compile vs_2_0 VtxShader_Wood();
      PixelShader  = compile ps_2_0 PixShader_Wood();
   }

}

//---------------------------------------------------------------------------------------

