//--------------------------------------------------------------//
// Toon Effect
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

// transformation matrices
float4x4 mtxWorld	: WORLD;  
float4x4 mtxView	: VIEW;
float4x4 mtxProj	: PROJECTION;
float4   gLight;    
float4   vecView;

texture baseTex;

sampler2D baseTex0 = sampler_state
{
	Texture   = <baseTex>;
	MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
	float3 Pos       : POSITION;
	float3 Normal    : NORMAL;
	float4 Diffuse   : COLOR0;
	float2 TexUV     : TEXCOORD0;

};

struct VS_OUTPUT
{
    float4 Pos        : POSITION;  
    float4 Diffuse    : COLOR0; 
    float4 Spec       : COLOR1;
    float2 TexUV      : TEXCOORD0;
};


//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------

VS_OUTPUT VtxShader_Toon( VS_INPUT In )
{
    VS_OUTPUT Out;
	
	// position xform
	In.Pos   = mul(float4(In.Pos, 1), (float4x3)mtxWorld);
	In.Pos   = mul(float4(In.Pos, 1), (float4x3)mtxView);
	Out.Pos  = mul(float4(In.Pos, 1), mtxProj);	
	
	float3 N = normalize( mul( In.Normal, (float3x3)mtxWorld ) );
	float3 V = normalize( vecView - Out.Pos );
	float3 L = normalize( gLight );
		
	// Specular lighting ----
	float3 R = reflect(L, N);
	R = normalize(R);
	
	float dpS = dot(V,R);
	dpS = max(0, dpS);
	Out.Spec = pow(dpS, 10);
	Out.Spec.a = 1.0f;
	// End Spec -------------
	
	// Diffuse lighting -----
	float dp = dot(-L, N);
	dp = max(0, dp);	
	
	Out.Diffuse = dp;
	Out.Diffuse.a = 1.0f;	
	// End Diffuse ----------	
	
	// Pass-thru texc coordinates
	Out.TexUV = In.TexUV;
		
    return Out;    
}

//--------------------------------------------------------------------------------------
// Pixel shader
//--------------------------------------------------------------------------------------

float4 PixShader_Toon( VS_OUTPUT In ) : COLOR
{ 
	//float4 baseColour  = tex2D(baseTex0,  In.TexUV);
	float4 baseColour = { 0.5,0.0,1.0,1.0 };

	In.Diffuse = floor(In.Diffuse.x*6)/6.0f;
	In.Spec    = floor(In.Spec.x*6)/6.0f;

	baseColour *= In.Diffuse.x;
	
	return baseColour + In.Spec * 1.0f;
}

//---------------------------------------------------------------------------------------

technique ToonEffect
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader_Toon( );
		PixelShader  = compile ps_2_0 PixShader_Toon( );          
    }
}
//---------------------------------------------------------------------------------------